// Clase DateTime provee al programador una forma sencilla de operar con fechas y hora.
// Copyright (C) Gammexane
// Gammexane@Yahoo.com.ar

// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#include "cdatetime.h"
/*##########################################################################################
 #                             Clase para manipulacion de Fechas                          # 
##########################################################################################*/
//--------------------------------     Constructores         -----------------------------//
cldate::cldate(){
	_SEP = '/'; 
}
cldate::cldate(const char *vfecha){
	*this = string(vfecha);
}
//--------------------------------  Operadores sobrecargados  -----------------------------//

// Operadores miembro
cldate& cldate::operator =(const cldate &vsdate){
	_date = vsdate.juliano();
	return *this;
}

cldate& cldate::operator =(int vidate){
	_date = vidate;
	return *this;
}

cldate& cldate::operator=(const string &vsdate){
	// Este calculo sirve para pasar de una fecha Gregoriana a un entero Juliano 
	STdate fecha = split(vsdate);
	_date = 
	( 1461 * ( fecha.year + 4800 + ( fecha.month - 14 ) / 12 ) ) / 4 +
	( 367  * ( fecha.month - 2 - 12 * ( ( fecha.month - 14 ) / 12 ) ) ) / 12 - 
	( 3    * ( ( fecha.year + 4900 + ( fecha.month - 14 ) / 12 ) / 100 ) ) / 4 +                    
	fecha.day - 32075;
	return *this;
}

cldate& cldate::operator++(){
	++_date;
	return *this;
}

cldate& cldate::operator--(){
	--_date;
	return *this;
}

// Operadores "Ajenos"
cldate operator+(const cldate &vfecha, int vsum){
	cldate vsfecha;
	vsfecha = vfecha.juliano() + vsum;
	return vsfecha;
} 

cldate operator-(const cldate &vfecha, int vsum){
	cldate vsfecha;
	vsfecha = vfecha.juliano() - vsum;
	return vsfecha;

}

int operator-(const cldate &vdate1, const cldate &vdate2){
	if (vdate1.juliano() > vdate2.juliano())
		return vdate1.juliano() - vdate2.juliano();
	else
		return vdate2.juliano() - vdate1.juliano();
} 

ostream& operator<<(ostream &o, const cldate &d){
	o << d.gregoriano();
	return o;
}

istream& operator>>(istream &i, cldate &d){
	string s;
	i >> s;
	d = s;
	return i;
}

//--------------------------------  Funciones Miembro  -----------------------------//
/*************************************************************************************
  Funcion Split: Se encarga de pasar a una estructura un string DD/MM/AAAA separandolo
                 en DD MM AAAA 
  Toma:     un string DD/MM/AAAA o D/M/AAAA
  Devuelve:	Una estructura fecha{int day, month, year}
*************************************************************************************/
cldate::STdate cldate::split(string vsdate) const{
	STdate fecha;
	sscanf(vsdate.c_str(), "%d%c%d%c%d", &fecha.day, &_SEP, &fecha.month, &_SEP, &fecha.year);
	return fecha;
}
/*************************************************************************************
  Funcion Gregoriano: Pasa un nro entero juliano a una fecha gregoriana  
  Toma:  la variable del objeto (_date)     
  Devuelve:	un string formateado en DD/MM/AAAA
*************************************************************************************/
string cldate::gregoriano() const{
	int L, n, i, j; 
    int d, m, y; 
    string vsdate;         
         
	L = _date + 68569;
    n = ( 4 * L) / 146097;
    L = L - ( 146097 * n + 3 ) / 4;
    i = ( 4000 * ( L + 1 ) ) / 1461001;
    L = L - ( 1461 * i ) / 4 + 31;
    j = ( 80 * L) / 2447;
    d = L - ( 2447 * j ) / 80;
    L = j / 11;
    m = j + 2 - ( 12 * L );
    y = 100 * ( n - 49 ) + i + L;
            
    char vfecha[11];
    sprintf(vfecha, "%2d%c%2d%c%2d", d, _SEP, m, _SEP, y);
    vsdate = vfecha;
    replace(vsdate.begin(), vsdate.end(), ' ', '0');
 
    return vsdate;
}
/*************************************************************************************
  Funcion format: (formato corto) formatea una fecha  
  Toma:     tres parametros enum (DD, MM, AAAA, AA) 
  Devuelve:	la fecha formateada segun los parametros ingresados
*************************************************************************************/

string cldate::format(DComp a, DComp b, DComp c) const{
	string fecha[4];
	string tmp = gregoriano();

	fecha[DD]   = tmp.substr(0,2);
	fecha[MM]   = tmp.substr(3,2);
	fecha[AA]   = tmp.substr(8,2);
	fecha[AAAA] = tmp.substr(6,4);
	
	return fecha[a] + _SEP + fecha[b] + _SEP + fecha[c];
}
/*************************************************************************************
  Funcion format: (formato Largo) formatea una fecha  
  Toma:     Un parametros enum (LN, LNG) 
  			LNG va a ser formato largo Ej: Viernes 7 de julio de 1978
  			LN  va a ser formato corto Ej  7 de jul de 1978	
  Devuelve:	la fecha formateada segun los parametros ingresados
*************************************************************************************/
string cldate::format(DLComp d) const{
	string tmp  = gregoriano();
	STdate dtmp = split(tmp);
	string fecha[4] = {tmp.substr(0,2), tmp.substr(3,2), tmp.substr(6,4), tmp.substr(8,2)};
	  
	static const string MES[12]  = {"Enero", "Febrero", "Marzo", "Abril", 
	                                "Mayo", "Junio", "Julio", "Agosto", 
	                                "Septiembre", "Octubre", "Noviembre", "Diciembre"};
	                                
	static const string mes[12] = {"Ene", "Feb", "Mar", "Abr", "May", "Jun", 
	                               "Jul", "Ago", "Sep", "Oct", "Nov", "Dic"};
	                               
    static const string dia[7] = {"Lunes", "Martes", "Miercoles", "Jueves", 
                                  "Viernes", "Sabado", "Domingo"};
    int vdia;
    
    vdia = _date%7;
    
    if (d == LN)
		return fecha[DD] + " de " + mes[dtmp.month - 1] + " de " + fecha[AAAA];	
	else
		return dia[vdia] + " " + fecha[DD] + " de " + MES[dtmp.month - 1] + " de " + fecha[AAAA];
}



/*##########################################################################################
 #                             Clase para manipulacion de Tiempo                          # 
##########################################################################################*/

//--------------------------------     Constructores         -----------------------------//
cltime::cltime(){
	_SEP = ':';
}

cltime::cltime(string t){
	*this = t;
}
//-------------------------------- Operadores sobrecarcados ------------------------------//

// Operadores Miembros
cltime& cltime::operator =(const string &vstime){
	int h, m, s;
	sscanf(vstime.c_str(), "%d%c%d%c%d", &h, &_SEP, &m, &_SEP, &s);
	// pasa la hora ingresada a segundo para facilitar los calculos
	_time = h*3600 + m*60 + s;
	return *this;
}

cltime& cltime::operator =(int vitime){
	_time = vitime;
	return *this;
} 

cltime& cltime::operator =(const cltime &vcltime){
	_time = vcltime.seconds();
	return *this;
}

cltime& cltime::operator++(){
	_time += 60;
	return *this;
}

cltime& cltime::operator--(){
	_time -= 60;
	return *this;
}

// Operadores "Ajenos"
ostream& operator<<(ostream &o, const cltime &t){
	o << t.hours();
	return o;
}

istream& operator>>(istream &i, cltime &t){
	string s;
	i >> s;
	t = s;
	return i;
}

cltime operator+(const cltime &vtime, int vsum){
	cltime vctime;
	vctime = vtime.seconds() + vsum;
	return vctime;
} 


//--------------------------------  Funciones Miembro  -----------------------------//
/*************************************************************************************
  Funcion hour: Transforma un entero en segundos a HH MM SS 
  Toma:     la variable _time del objeto 
  Devuelve:	un string HH:MM:SS
*************************************************************************************/
string  cltime::hours() const{
	STtime time;
	time.hour =  _time / 3600;
	time.min  = (_time-(time.hour*3600)) / 60;
	time.sec  = _time - time.hour*3600 - time.min*60;
	char tmp_time[9];
	sprintf(tmp_time, "%2d%c%2d%c%2d", time.hour, _SEP, time.min, _SEP, time.sec);
	for(int i = 0; i < 8; ++i)
		if(tmp_time[i] == ' ') tmp_time[i] = '0';
	string vtime = tmp_time;
	return vtime;
}
